<?php
/*
 * Plugin Name: JSM's file_get_contents() Shortcode
 * Text Domain: wp-file-get-contents
 * Domain Path: /languages
 * Plugin URI: https://surniaulula.com/extend/plugins/wp-file-get-contents/
 * Assets URI: https://jsmoriss.github.io/wp-file-get-contents/assets/
 * Author: JS Morisset
 * Author URI: https://surniaulula.com/
 * License: GPLv3
 * License URI: https://www.gnu.org/licenses/gpl.txt
 * Description: A WordPress shortcode for PHP's file_get_contents()
 * Requires PHP: 7.2.5
 * Requires At Least: 5.5
 * Tested Up To: 6.2.0
 * Version: 2.4.0
 *
 * Version Numbering: {major}.{minor}.{bugfix}[-{stage}.{level}]
 *
 *      {major}         Major structural code changes and/or incompatible API changes (ie. breaking changes).
 *      {minor}         New functionality was added or improved in a backwards-compatible manner.
 *      {bugfix}        Backwards-compatible bug fixes or small improvements.
 *      {stage}.{level} Pre-production release: dev < a (alpha) < b (beta) < rc (release candidate).
 *
 * Copyright 2012-2023 Jean-Sebastien Morisset (https://surniaulula.com/)
 */

if ( ! defined( 'ABSPATH' ) ) {

	die( 'These aren\'t the droids you\'re looking for.' );
}

if ( ! class_exists( 'WPFGC' ) ) {

	class WPFGC {

		private $cache_disabled = false;	// Signal to clear and re-create the cache object.

		private $shortcode_names = array(
			'wp-file-get-contents',
			'wpfgc',
		);

		private static $instance = null;	// WPFGC class object.

		public function __construct() {

			add_action( 'plugins_loaded', array( $this, 'init_textdomain' ) );

			/*
			 * Allow for an additional custom shortcode name.
			 */
			if ( defined( 'WPFGC_SHORTCODE_NAME' ) && WPFGC_SHORTCODE_NAME ) {

				if ( ! in_array( WPFGC_SHORTCODE_NAME ) ) {	// Just in case.

					$this->shortcode_names[] = WPFGC_SHORTCODE_NAME;
				}
			}

			if ( is_admin() ) {

				add_action( 'save_post', array( $this, 'clear_post_cache' ), 10 );
			}

			$this->check_wpautop();

			$this->add_shortcodes();
		}

		public static function &get_instance() {

			if ( null === self::$instance ) {

				self::$instance = new self;
			}

			return self::$instance;
		}

		public function init_textdomain() {

			load_plugin_textdomain( 'wp-file-get-contents', false, 'wp-file-get-contents/languages/' );
		}

		public function check_wpautop() {

			$default_priority = 10;

			foreach ( array( 'get_the_excerpt', 'the_excerpt', 'the_content' ) as $filter_name ) {

				$filter_priority = has_filter( $filter_name, 'wpautop' );	// Can return a priority of 0.

				if ( false !== $filter_priority && $filter_priority > $default_priority ) {

					remove_filter( $filter_name, 'wpautop' );

					add_filter( $filter_name, 'wpautop' , $default_priority );
				}
			}
		}

		public function add_shortcodes() {

			foreach ( $this->shortcode_names as $name ) {

        			add_shortcode( $name, array( $this, 'do_shortcode' ) );
			}
		}

		public function remove_shortcodes() {

			foreach ( $this->shortcode_names as $name ) {

				remove_shortcode( $name );
			}
		}

		public function do_shortcode( $atts = array(), $content = null, $tag = '' ) {

			if ( ! is_array( $atts ) ) {	// Empty string if no shortcode attributes.

				$atts = array();
			}

			/*
			 * Determine the url / file name to retrieve.
			 */
			if ( ! empty( $atts[ 'url' ] ) && preg_match( '/^https?:\/\//', $atts[ 'url' ] ) ) {

				$do_url = $atts[ 'url' ];

			} elseif ( ! empty( $atts[ 'url' ] ) && preg_match( '/^file:\/\//', $atts[ 'url' ] ) ) {

				$do_url = trailingslashit( WP_CONTENT_DIR ) . preg_replace( '/(^file:\/\/|\.\.)/', '', $atts[ 'url' ] );

			} elseif ( ! empty( $atts[ 'file' ] ) ) {

				$do_url = trailingslashit( WP_CONTENT_DIR ) . preg_replace( '/(^\/+|\.\.)/', '', $atts[ 'file' ] );

			} else {

				$error_msg = sprintf( __( '%1$s or %2$s shortcode attribute missing.', 'wp-file-get-contents' ), '<code>url</code>', '<code>file</code>' );

				return '<p><strong>' . __CLASS__ . ': ' . $error_msg . '</strong></p>';
			}

			$cache_salt  = __METHOD__ . '_' . $this->get_atts_salt( $atts );
			$cache_id    = __CLASS__ . '_' . md5( $cache_salt );
			$cache_exp   = isset( $atts[ 'cache' ] ) ? (int) $atts[ 'cache' ] : 3600;
			$do_body     = isset( $atts[ 'body' ] ) ? self::get_bool( $atts[ 'body' ] ) : true;
			$do_esc_html = isset( $atts[ 'esc_html' ] ) ? self::get_bool( $atts[ 'esc_html' ] ) : false;
			$do_filter   = isset( $atts[ 'filter' ] ) ? sanitize_text_field( $atts[ 'filter' ] ) : 'wpfgc_content';
			$do_pre      = isset( $atts[ 'pre' ] ) ? self::get_bool( $atts[ 'pre' ] ) : false;
			$do_utf8     = isset( $atts[ 'utf8' ] ) ? self::get_bool( $atts[ 'utf8' ] ) : true;

			if ( $this->cache_disabled ) {	// Signal to clear and re-create the cache object.

				delete_transient( $cache_id );

			} elseif ( $cache_exp ) {

				$content = get_transient( $cache_id );

				if ( false !== $content ) {

					return $this->format_content( $content, $atts );
				}
			}

			$content = file_get_contents( $do_url );

			/*
			 * Maybe keep only <body></body> content (default is true).
			 */
			if ( $do_body && false !== stripos( $content, '<body' ) ) {

				$content = preg_replace( '/^.*<body[^>]*>(.*)<\/body>.*$/is', '$1', $content );
			}

			/*
			 * Maybe escape HTML characters (default is false).
			 */
			if ( $do_esc_html ) {

				$content = esc_html( $content );
			}

			/*
			 * Maybe convert UTF-8 to HTML entities (default is true).
			 */
			if ( $do_utf8 && function_exists( 'mb_convert_encoding' ) ) {

				$content = mb_convert_encoding( $content, $to_encoding = 'HTML-ENTITIES', $from_encoding = 'UTF-8' );
			}

			/*
			 * Mrap content in pre tags (default is false).
			 */
			if ( $do_pre ) {

				$content = '<pre>' . "\n" . $content . '</pre>' . "\n";
			}

			/*
			 * Apply content filter name (default is 'wpfgc_content').
			 */
			if ( $do_filter ) {

				$this->remove_shortcodes();	// Just in case, to prevent recursion.

				$content = apply_filters( $do_filter, $content );

				$this->add_shortcodes();
			}

			/*
			 * Maybe cache the content (default is 1 hour).
			 */
			if ( $cache_exp ) {

				set_transient( $cache_id, $content, $cache_exp );	// Save rendered content.
			}

			return $this->format_content( $content, $atts );
		}

		/*
		 * Hooked to the 'save_post' action.
		 */
		public function clear_post_cache( $post_id, $rel_id = false ) {

			switch ( get_post_status( $post_id ) ) {

				case 'draft':
				case 'pending':
				case 'future':
				case 'private':
				case 'publish':
				case 'expired':

					$is_admin = is_admin();
					$post_obj = get_post( $post_id, OBJECT, 'raw' );

					if ( isset( $post_obj->post_content ) ) {	// Just in case.

						foreach ( $this->shortcode_names as $name ) {

							if ( false !== stripos( $post_obj->post_content, '[' . $name ) ) {

								$this->cache_disabled = true;	// Signal to clear and re-create the cache object.

								$content = do_shortcode( $post_obj->post_content );

								break;	// Stop after first shortcode match.
							}
						}
					}

					break;
			}

			return $post_id;
		}

		private function get_atts_salt( $atts ) {

			unset( $atts[ 'cache' ], $atts[ 'class' ], $atts[ 'more' ] );	// Not relevant for cache salt.

			return implode( '_', array_map( function( $k, $v ) { return $k . ':' . $v; }, array_keys( $atts ), array_values( $atts ) ) );
		}

		private function format_content( $content, $atts ) {

			$do_class = empty( $atts[ 'class' ] ) ? '' : ' ' . esc_attr( $atts[ 'class' ] );		// Optional css class names.
			$do_more  = isset( $atts[ 'more' ] ) ? self::get_bool( $atts[ 'more' ] ) : true;		// Add more link (default is true).

			/*
			 * Maybe add a more link (default is true).
			 */
			if ( $do_more && ! is_singular() ) {

				global $post;

				$parts = get_extended( $content );

				if ( $parts[ 'more_text' ] ) {

					$more_link = sprintf( ' <a href="%s#more-{%s}" class="more-link">%s</a>', get_permalink(), $post->ID, $parts[ 'more_text' ] );

					$more_link = apply_filters( 'the_content_more_link', $more_link, $parts[ 'more_text' ] );

					$content = $parts[ 'main' ] . $more_link;

				} else {

					$content = $parts[ 'main' ];
				}
			}

			$content = '<div class="wp_file_get_contents wpfgc' . $do_class . '">' . "\n" . $content . '</div><!-- .wp_file_get_contents -->' . "\n";

			return $content;
		}

		/*
		 * Converts string to boolean.
		 */
		private static function get_bool( $mixed ) {

			return is_string( $mixed ) ? filter_var( $mixed, FILTER_VALIDATE_BOOLEAN ) : (bool) $mixed;
		}
	}

        WPFGC::get_instance();
}
